<?php
/**
 * This file (SimpleHTML.php) is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This file (SimpleHTML.php) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this file (SimpleHTML.php); if not, visit http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

/**
 * A class to treat an HTML document as a PHP object
 *
 * PHP version 5
 *
 * @copyright 2014 Ben Dusinberre <ben@dq5studios.com>
 * @license http://www.gnu.org/licenses/lgpl.html LGPL, v3
 */
class SimpleHTML extends SimpleXMLElement
{


    /**
     * Public Methods
     */


    /**
     * The magic method that is called when an element that doesn't have its
     * own method is added. Creates and adds objects to the node tree based
     * on supplied arguments.
     *
     * @param string $element   Element name
     * @param array  $arguments Element attributes
     *
     * @return SimpleHTML Newly created node
     */
    public function __call($element, $arguments)
    {
        $fArgs = array(null, null);
        foreach ($arguments as $index => $arg) {
            $fArgs[$index] = $arg;
        }
        $newnode = self::factory($element, $fArgs[0], $fArgs[1]);
        if (isset($this)) {
            $newnode = $this->addChild($newnode);
        }
        return $newnode;
    }

    /**
     * Converts the object into a valid XHTML 1.0 Strict document with indenting.
     * If user agent does not accept application/xhtml+xml, the XML prolog is removed.
     * If an html fragment (i.e. &lt;html&gt; is not root), DOCTYPE and XML prolog are not added.
     *
     * @return string XHTML document
     */
    public function __toString()
    {
        $removeProlog = false;
        if ($this->getName() == "html") {
            $doctype = DOMImplementation::createDocumentType(
                "html",
                "-//W3C//DTD XHTML 1.0 Strict//EN",
                "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"
            );
            $doc = DOMImplementation::createDocument(null, null, $doctype);
            if (SimpleHTML::detectProperMimeType() == "text/html") {
                // Having an xml prolog causes IE to go into quirks mode and forget how to render the internet
                $removeProlog = true;
            }
        } else {
            $doc = DOMImplementation::createDocument();
            $removeProlog = true;  // If it's a fragment, there's no need for the prolog
        }
        $doc->formatOutput = true;
        $domnode = dom_import_simplexml($this);
        $domnode = $doc->importNode($domnode, true);
        $domnode = $doc->appendChild($domnode);
        $return = $doc->saveXML();

        if ($removeProlog) {
            $return = str_replace("<?xml version=\"1.0\"?>\n", "", $return);
        }

        return $return;
    }

    /**
     * Create an <a> element
     *
     * If a single string argument is given, it is the href attribute instead.
     * If an argument is an array, they are attributes.
     *
     * @param mixed  $inner Link contents
     * @param string $href  Destination
     *
     * @return SimpleHTML Newly created &lt;a&gt; element
     */
    public function a($inner, $href = null)
    {
        if (is_array($href)) {
            return self::__call("a", array($inner, $href));
        }
        if (is_array($inner)) {
            return self::__call("a", array($inner));
        }
        if (is_null($href)) {
            $href = $inner;
            $inner = "";
        }
        $cont = (is_object($inner)?"":$inner);
        $return = $this->a($cont, array("href" => $href));
        if (is_object($inner)) {
            $return->addChild($inner);
        }
        return $return;
    }

    /**
     * Overloaded SimpleXMLElement function to allow adding an existing object
     *
     * NOTE: A copy of $node is added, not the original
     *
     * @param mixed  $node     Node to be added
     * @param string $contents Value of node
     *
     * @return SimpleHTML Newly created node
     */
    public function addChild($node, $contents = null)
    {
        if (is_string($node)) {
            return parent::addChild($node, (is_null($contents) ? null : htmlentities($contents)));
        }
        $node1 = dom_import_simplexml($this);
        $dom_sxe = dom_import_simplexml($node);
        $node2 = $node1->ownerDocument->importNode($dom_sxe, true);
        return simplexml_import_dom($node1->appendChild($node2), "SimpleHTML");
    }

    /**
     * Create an <img> inside an <a>
     *
     * @param string $href  Link destination
     * @param string $src   Image source
     * @param string $alt   Alternate text
     * @param string $title Title text
     *
     * @return SimpleHTML Newly created &lt;a&gt; element
     */
    public function aimg($href, $src, $alt = "", $title = null)
    {
        $img = self::img($src, $alt, $title);
        $link = self::a($img, $href);
        return $link;
    }

    /**
     * Appends text to the contents of an element
     *
     * @param string $text Text to append
     *
     * @return SimpleHTML Updated node
     */
    public function appendText($text)
    {
        $node = dom_import_simplexml($this);
        return $node->appendChild(new DOMText($text));
    }

    /**
     * Create a checkbox
     *
     * @param string  $name    Name value
     * @param string  $value   Default value
     * @param string  $id      Id value
     * @param boolean $checked Checked
     * @param array   $extra   Extra attributes
     *
     * @return SimpleHTML Newly created &lt;input&gt; element
     */
    public function checkbox($name, $value = "", $id = null, $checked = false, $extra = null)
    {
        if ($checked) {
            if (is_null($extra)) {
                $extra = array("checked" => "checked");
            } else {
                $extra["checked"] = "checked";
            }
        }
        return self::input($name, $value, $id, "checkbox", $extra);
    }

    /**
     * Create a <link> element for a CSS file
     *
     * If the first argument is an array, a &lt;link&gt; is created for each one
     * If not provided, the currently running script name is used
     *
     * @param mixed  $href  CSS file(s) to link
     * @param string $title Title of link
     *
     * @return SimpleHTML Last created &lt;link&gt; element
     */
    public function css($href = null, $title = null)
    {
        if (is_null($href)) {
            $href = basename($_SERVER["SCRIPT_FILENAME"], ".php") . ".css";
        }
        if (!is_array($href)) {
            $href = array($href);
        }
        foreach ($href as $h) {
            $attr = array("rel" => "stylesheet", "type" => "text/css", "href" => $h);
            if (!is_null($title)) {
                $attr["title"] = $title;
            }
            $return = $this->link($attr);
        }
        return $return;
    }

    /**
     * Create a file upload input
     *
     * @param string $name  Name value
     * @param string $value Default value
     * @param string $id    Id value
     * @param array  $extra Extra attributes
     *
     * @return SimpleHTML Newly created &lt;input&gt; element
     */
    public function file($name, $value = "", $id = null, $extra = null)
    {
        return self::input($name, $value, $id, "file", $extra);
    }

    /**
     * Create a <form> element
     *
     * @param string $method Form submission method
     * @param string $action Form submission destination
     * @param array  $extra  Extra attributes
     *
     * @return SimpleHTML Newly created &lt;&gt&gt; element
     */
    public function form($method = "get", $action = null, $extra = null)
    {
        if (is_array($method)) {
            return self::__call("form", array($method));
        }
        if (is_null($action)) {
            $action = $_SERVER["PHP_SELF"];
        }
        $attr = array("method" => $method, "action" => $action);
        if (!is_null($extra)) {
            $attr = array_merge($attr, $extra);
        }
        $return = $this->form($attr);
        return $return;
    }

    /**
     * Create a hidden input
     *
     * @param string $name  Name value
     * @param string $value Default value
     * @param string $id    Id value
     * @param array  $extra Extra attributes
     *
     * @return SimpleHTML Newly created &lt;input&gt; element
     */
    public function hidden($name, $value = "", $id = null, $extra = null)
    {
        return self::input($name, $value, $id, "hidden", $extra);
    }

    /**
     * Create an <img> element
     *
     * If the first argument is an array, they are attributes.
     *
     * @param string $src   Image source
     * @param string $alt   Alternate text
     * @param string $title Title text
     *
     * @return SimpleHTML Newly created &lt;img&gt; element
     */
    public function img($src, $alt = "", $title = null)
    {
        if (is_array($src)) {
            return self::__call("img", array($src));
        }
        $attr = array("src" => $src, "alt" => $alt, "title" => (is_null($title)?$alt:$title));
        $return = $this->img($attr);
        return $return;
    }

    /**
     * Create an <input> element
     *
     * If the first argument is an array, they are attributes.
     *
     * @param string $name  Name value
     * @param string $value Default value
     * @param string $id    Id value
     * @param string $type  Input type
     * @param array  $extra Extra attributes
     *
     * @return SimpleHTML Newly created &lt;input&gt; element
     */
    public function input($name, $value = "", $id = null, $type = "text", $extra = null)
    {
        if (is_array($name)) {
            return self::__call("input", array($name));
        }
        if (is_null($id)) {
            $id = $name;
        }
        $attr = array("type" => $type, "name" => $name, "id" => $id, "value" => $value);
        if (!is_null($extra)) {
            $attr = array_merge($attr, $extra);
        }
        $return = $this->input($attr);
        return $return;
    }

    /**
     * Create a <script> element for a Javascript file
     *
     * If the first argument is an array, a &lt;script&gt; is created for each one
     * If not provided, the currently running script name is used
     *
     * @param mixed $src Javascript file(s) to link
     *
     * @return SimpleHTML Last created &lt;script&gt; element
     */
    public function js($src = null)
    {
        if (is_null($src)) {
            $src = basename($_SERVER["SCRIPT_FILENAME"], ".php") . ".js";
        }
        if (!is_array($src)) {
            $src = array($src);
        }
        foreach ($src as $s) {
            $attr = array("type" => "text/javascript", "src" => $s);
            $return = $this->script($attr);
        }
        return $return;
    }

    /**
     * Create a <label> element
     *
     * If an argument is an array, they are attributes.
     *
     * @param string $contents Label contents
     * @param string $for      For value
     *
     * @return SimpleHTML Newly created &lt;label&gt; element
     */
    public function label($contents, $for = null)
    {
        if (is_array($for) || is_array($contents)) {
            return self::__call("label", array($contents, $for));
        }
        if (is_null($for)) {
            $for = array();
        } else {
            $for = array("for"=>$for);
        }
        $return = $this->label($contents, $for);
        return $return;
    }

    /**
     * Create a password input
     *
     * @param string $name  Name value
     * @param string $value Default value
     * @param string $id    Id value
     * @param array  $extra Extra attributes
     *
     * @return SimpleHTML Newly created &lt;input&gt; element
     */
    public function password($name, $value = "", $id = null, $extra = null)
    {
        return self::input($name, $value, $id, "password", $extra);
    }

    /**
     * Create a radio input
     *
     * @param string  $name    Name value
     * @param string  $value   Default value
     * @param string  $id      Id value
     * @param boolean $checked Checked
     * @param array   $extra   Extra attributes
     *
     * @return SimpleHTML Newly created &lt;input&gt; element
     */
    public function radio($name, $value = "", $id = null, $checked = false, $extra = null)
    {
        if ($checked) {
            if (is_null($extra)) {
                $extra = array("checked" => "checked");
            } else {
                $extra["checked"] = "checked";
            }
        }
        return self::input($name, $value, $id, "radio", $extra);
    }

    /**
     * Create a reset input
     *
     * @param string $name  Name value
     * @param string $value Default value
     * @param string $id    Id value
     * @param array  $extra Extra attributes
     *
     * @return SimpleHTML Newly created &lt;input&gt; element
     */
    public function reset($name = "reset", $value = "Reset", $id = null, $extra = null)
    {
        return self::input($name, $value, $id, "reset", $extra);
    }

    /**
     * Turn an array into a <select> element
     *
     * Each row of the array will correspond to an &lt;option&gt; element.
     * If an array element is also an array, the first two elements are
     * used as the contents and value.  Otherwise it is used
     * as the both.
     *
     * @param array  $options  Select options
     * @param array  $selected Option to default as selected
     * @param array  $name     Name attribute
     * @param string $id       Id attribute
     * @param array  $extra    Extra attributes
     *
     * @return SimpleHTML Newly created &lt;select&gt; element
     */
    public function selectize($options, $selected = null, $name = null, $id = null, $extra = null)
    {
        if (isset($this)) {
            $select = $this->select();
        } else {
            $select = SimpleHTML::factory("select");
        }
        if (!is_null($name)) {
            $select->addAttribute("name", $name);
            if (is_null($id)) {
                $id = $name;
            }
        }
        if (!is_null($id)) {
            $select->addAttribute("id", $id);
        }
        if (is_array($extra)) {
            foreach ($extra as $newAttrName => $newAttrValue) {
                $select->addAttribute($newAttrName, $newAttrValue);
            }
        }
        foreach ($options as $option) {
            if (is_array($option)) {
                $o = $select->option($option[0], array("value" => $option[1]));
                $v = $option[1];
            } else {
                $o = $select->option($option, array("value" => $option));
                $v = $option;
            }
            if (!is_null($selected) && ((string) $v === (string) $selected)) {
                $o["selected"]  = "selected";
            }
        }
        return $select;
    }

    /**
     * Create a submit input
     *
     * @param string $name  Name value
     * @param string $value Default value
     * @param string $id    Id value
     * @param array  $extra Extra attributes
     *
     * @return SimpleHTML Newly created &lt;input&gt; element
     */
    public function submit($name = "submit", $value = "Submit", $id = null, $extra = null)
    {
        return self::input($name, $value, $id, "submit", $extra);
    }

    /**
     * Turn an array into a <table> element
     *
     * Each row of the array will correspond to a &lt;tr&gt; element.
     * Each array element will correspond to a &lt;td&gt; element.
     * If the array element is an array, the first two elements are
     * used as the contents and attributes.  Otherwise it is used
     * as the contents
     *
     * @param array  $body    Table contents
     * @param array  $head    Header row
     * @param array  $foot    Footer data
     * @param string $caption Table caption
     *
     * @return SimpleHTML Newly created &lt;table&gt; element
     */
    public function tableize($body, $head = null, $foot = null, $caption = null)
    {
        if (isset($this)) {
            $table = $this->table();
        } else {
            $table = SimpleHTML::factory("table");
        }
        if (!is_null($caption)) {
            $table->caption($caption);
        }
        if (!is_null($head)) {
            $tr = $table->thead()->tr();
            foreach ($head as $td) {
                if (is_array($td)) {
                    $tr->td($td[0], $td[1]);
                } else {
                    $tr->td($td);
                }
            }
        }
        if (!is_null($foot)) {
            $tr = $table->tfoot()->tr();
            foreach ($foot as $td) {
                if (is_array($td)) {
                    $tr->td($td[0], $td[1]);
                } else {
                    $tr->td($td);
                }
            }
        }
        $table->tbody();
        foreach ($body as $row) {
            $tr = $table->tbody->tr(array("class"=>(($odd = !$odd)?"rowodd":"roweven")));
            foreach ($row as $td) {
                if (is_array($td)) {
                    $tr->td($td[0], $td[1]);
                } else {
                    $tr->td($td);
                }
            }
        }
        return $table;
    }

    /**
     * Create a text input
     *
     * @param string $name  Name value
     * @param string $value Default value
     * @param string $id    Id value
     * @param array  $extra Extra attributes
     *
     * @return SimpleHTML Newly created &lt;input&gt; element
     */
    public function text($name, $value = "", $id = null, $extra = null)
    {
        return self::input($name, $value, $id, "text", $extra);
    }


    /**
     * Static Methods
     */


    /**
     * See if the requester supports the XHTML mime type
     *
     * Checks the HTTP_ACCEPT header and if the User Agent prefers it
     * the MIME type <code>application/xhtml+xml</code> is returned
     * instead of <code>text/html</code>
     *
     * @see http://www.workingwith.me.uk/articles/scripting/mimetypes Based on code from Neil Crosby
     *
     * @return string The appropriate MIME type
     */
    public static function detectProperMimeType()
    {
        $mime = "text/html";
        if (isset($_SERVER["HTTP_USER_AGENT"]) && stristr($_SERVER["HTTP_USER_AGENT"], "W3C_Validator")) {
            // The W3C Validator doesn't have properly set up ACCEPT headers, IRONY!
            $mime = "application/xhtml+xml";
        } elseif (isset($_SERVER["HTTP_ACCEPT"]) && stristr($_SERVER["HTTP_ACCEPT"], "application/xhtml+xml")) {
            if (preg_match("/application\/xhtml\+xml[\s]*;[\s]?q=([0-1]{0,1}\.\d{0,4})/i",
             $_SERVER["HTTP_ACCEPT"], $matches)) {
                $xhtml_q = $matches[1];
                if (preg_match("/text\/html[\s]*;[\s]?q=([0-1]{0,1}\.\d{0,4})/i",
                 $_SERVER["HTTP_ACCEPT"], $matches)) {
                    $html_q = $matches[1];
                    if ( (float) $html_q < (float) $xhtml_q) {
                        $mime = "application/xhtml+xml";
                    }
                }
            } else {
                $mime = "application/xhtml+xml";
            }
        }
        return $mime;
    }

    /**
     * Set the Content Type header to the appropriate MIME type
     */
    public static function sendMimeType()
    {
        $mimeType = self::detectProperMimeType();
        header("Content-Type: " . $mimeType);
        header("Vary: Accept");  // Lets the browser know HTTP_ACCEPT can cause changes in the server's response
    }

    /**
     * This is the function that turns out the SimpleHTML object
     *
     * The first attribute is used as the element. The second and third attributes
     * are used as contents or attributes of the new element depending on type.
     * As SimpleXML is UTF8 only, non UTF8 characters will be removed or replaced
     * as best as possible.
     *
     * @param string $element Base node
     * @param mixed  $a       Contents
     * @param mixed  $b       Attributes
     *
     * @return SimpleHTML Newly created node
     */
    public static function factory($element = "html", $a = null, $b = null)
    {
        $children = null;
        $contents = null;
        $attributes = array();
        foreach (array($a, $b) as $p) {
            if (is_string($p) || is_int($p)) {
                $contents = $p;
            }
            if (is_array($p) && !empty($p)) {
                foreach ($p as $attr => $value) {
                    if (in_array(strtolower($attr), array("id", "for"))) {  // This should probably go somewhere else
                        $value = preg_replace("/[^a-zA-Z0-9\-_:.]/", "-", $value);
                    }
                    $attributes[$attr] = $value;
                }
            }
            if (is_object($p)) {
                $children = $p;
            }
        }

        if (!is_null($contents)) {
            // Encode any HTML entities (<, >, etc)
            $contents = htmlentities(htmlspecialchars_decode(htmlentities($contents)), ENT_COMPAT, "UTF-8", true);
            // Replace any MS Word nonsense
            $contents = preg_replace("/([\x80-\xFF])/e", "chr(0xC0|ord('\\1')>>6).chr(0x80|ord('\\1')&0x3F)", $contents);
            // Convert anything left to the nearest UTF-8 equivilent
            $contents = iconv("UTF-8", "UTF-8//TRANSLIT", $contents);
            $object = "<" . $element . ">" . $contents . "</" . $element . ">";
        } elseif ($element[0] != "<") {
            $object = "<" . $element . "/>";
        } else {
            $object = $element;
        }

        $newnode = simplexml_load_string($object, "SimpleHTML");

        if ($newnode === false) {
            // The element failed to be created, lets return null
            return null;
        }

        foreach ($attributes as $attr => $value) {
            if (!is_null($value)) {
                $newnode->addAttribute($attr, $value);
            }
        }
        if (!is_null($children)) {
            $newnode->addChild($children);
        }
        return $newnode;
    }

}


?>